#include <iostream>
#include <cmath>
#include <cstdlib>
#include "rbf.h"

void get_grid_dimensions(int n, double *xy, double &xmin, double &xmax, double &ymin, double &ymax);

// text input:
//  # of points (integer)
//  x0 y0 f0 (3 doubles)
//  x1 y1 f1 (3 doubles)
//  ...

// text output:
//  mesh size (integer)
//  xmin xmax (2 doubles)
//  ymin ymax (2 doubles)
//  f(xmin, ymin) (double)
//  f(xmin+dx, ymin)
//  ...
//  f(xmax-dx, ymax) (double)
//  f(xmax, ymax) (double)

int main()
{
   // read in the data
   int n;
   std::cin>>n;
   if(n<3){
      std::cerr<<"Need to specify at least 3 data points"<<std::endl;
      return 1;
   }
   double *xy=(double*)std::malloc(2*n*sizeof(double));
   double *f=(double*)std::malloc(n*sizeof(double));
   for(int i=0; i<n; ++i){
      std::cin>>xy[2*i]>>xy[2*i+1]>>f[i];
   }

   // do the RBF solve
   double *coeffs=(double*)std::malloc((n+3)*sizeof(double));
   solveRBF(n, xy, f, coeffs);

   // write out the interpolated values
   const int meshn=50;
   double xmin, xmax, ymin, ymax;
   get_grid_dimensions(n, xy, xmin, xmax, ymin, ymax);
   double dx=(xmax-xmin)/(meshn-1), dy=(xmax-xmin)/(meshn-1);
   std::cout<<meshn<<"\n"<<xmin<<" "<<xmax<<"\n"<<ymin<<" "<<ymax<<std::endl;
   for(int j=0; j<meshn; ++j){
      for(int i=0; i<meshn; ++i){
         std::cout<<evalRBF(n, xy, coeffs, xmin+i*dx, ymin+j*dy)<<std::endl;
      }
   }

   std::free(xy);
   std::free(f);
   std::free(coeffs);
   return 0;
}

void get_grid_dimensions(int n, double *xy, double &xmin, double &xmax, double &ymin, double &ymax)
{
   // first find the actual extent of the data points
   xmin=xmax=xy[0];
   ymin=ymax=xy[1];
   for(int i=1; i<n; ++i){
      if(xy[2*i]<xmin) xmin=xy[2*i];
      else if(xy[2*i]>xmax) xmax=xy[2*i];
      if(xy[2*i+1]<ymin) ymin=xy[2*i+1];
      else if(xy[2*i+1]>ymax) ymax=xy[2*i+1];
   }
   // then come up with a mesh width that's a bit larger
   double width=xmax-xmin;
   if(ymax-ymin>width) width=ymax-ymin;
   width*=1.1;
   // and finally set the grid limits to be centred on the bounding box of the data points
   double xmid=0.5*(xmin+xmax), ymid=0.5*(ymin+ymax);
   xmin=xmid-0.5*width;
   xmax=xmid+0.5*width;
   ymin=ymid-0.5*width;
   ymax=ymid+0.5*width;
}

